#include <cstdio>
#include <cstring>
#include <iostream>
#include <vector>

#include <filter/filter/low_pass_filter.hpp>
#include <filter/window/hamming_window.hpp>

#include <cxxopts.hpp>

const std::size_t BUFFER_SIZE = 256;
const std::size_t FILTER_SIZE = 64;

using namespace Filter;

int main(int argc, char** argv) {
  // parse arguments
  cxxopts::Options options("filter", "filter signals; stdin->stdout");
  options.add_options()("c,cutoff", "Cutoff frequency in Hz.", cxxopts::value<size_t>()->no_implicit_value())("s,sample_rate", "Sample rate of input in Hz.",
                                                                                                              cxxopts::value<size_t>()->no_implicit_value());

  cxxopts::ParseResult result;
  try {
    result = options.parse(argc, argv);
  } catch (cxxopts::exceptions::exception e) {
    std::cerr << e.what() << std::endl;
    return -1;
  }

  if (result.count("cutoff") + result.count("sample_rate") != 2) {
    std::cerr << options.help() << std::endl;
    return -1;
  }

  // prepare pipes for binary data
  std::freopen(nullptr, "rb", stdin);
  if (std::ferror(stdin)) {
    std::cerr << "Failed to reopen stdin: " << std::strerror(errno) << std::endl;
    return errno;
  }

  std::freopen(nullptr, "wb", stdout);
  if (std::ferror(stdout)) {
    std::cerr << "Failed to reopen stdout: " << std::strerror(errno) << std::endl;
    return errno;
  }

  // create low pass filter
  LowPassFilter lpf(FILTER_SIZE, result["cutoff"].as<std::size_t>(), result["sample_rate"].as<std::size_t>(), HammingWindow(FILTER_SIZE).getWindow());

  lpf.Dump("filter.dmp");
  HammingWindow(FILTER_SIZE).Dump("window.dmp");

  // read data
  std::vector<float> inputBuffer(BUFFER_SIZE);

  std::vector<float> outputBuffer(BUFFER_SIZE);

  std::size_t len;

  while ((len = std::fread(inputBuffer.data(), sizeof(inputBuffer[0]), BUFFER_SIZE, stdin)) > 0) {
    if (std::ferror(stdin) && !std::feof(stdin)) {
      std::cerr << "Failed to read from stdin: " << std::strerror(errno) << std::endl;
      return errno;
    }

    lpf.ConvolveIQ(inputBuffer, outputBuffer);

    std::fwrite(outputBuffer.data(), sizeof(float), BUFFER_SIZE, stdout);
  }

  return 0;
}